/********************************************************************
     CNOVELL.C

     Routines to determine the user name, connection number, 
          and workstation node from Novell.

     by Richard S. Carter, Ph.D.
     June 3, 1994

********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <string.h>
#include <ctype.h>

#include <pro_ext.h>

union  _REGS  inregs, outregs;
struct _SREGS segregs;

struct {
     int       length ;
     char      function;
     char      station ;
} RequestPacket ;

struct {
     int       length ;
     char      uniqueid[4] ;
     int       type ;
     char      objectname[48] ;
     char      logtime[8] ;
} ReplyPacket ;

struct {
     int       length ;
     char      network[4] ;
     char      host[6] ;
     char      socket[2] ;
} map ;


/**************************************************************************
**
**   Get Novell User Name
**
***************************************************************************/
void FAR user_name(ParamBlk FAR *parm)

{
     int station,physical ;

     _segread(&segregs) ;
     segregs.es = segregs.ds ;

//   Get Station  
         
     inregs.x.ax=(0xdc00) ;
     _intdosx(&inregs, &outregs, &segregs) ;
     station=(outregs.x.ax & 0xff) ;
     
//   Get Physical Connection 

     inregs.x.ax=(0xee00) ;
     _intdosx(&inregs, &outregs, &segregs) ;
     physical=outregs.x.ax ;

//   Get username

     inregs.x.si=(int)&RequestPacket ;
     inregs.x.di=(int)&ReplyPacket ;
     inregs.x.ax= (0xe300) ;
     ReplyPacket.length=sizeof(ReplyPacket) ;
     RequestPacket.length=sizeof(RequestPacket) ;
     RequestPacket.function=22 ;
     RequestPacket.station=station ;
     _intdosx(&inregs, &outregs, &segregs) ;

     inregs.x.si=(int)&RequestPacket ;
     inregs.x.di=(int)&map ;
     inregs.x.ax=0xe300 ;
     RequestPacket.length=sizeof(RequestPacket) ;
     RequestPacket.function=19 ;
     RequestPacket.station=station ;
     map.length=sizeof(map) ;
     _intdosx(&inregs, &outregs, &segregs) ;

     physical=map.host[5] ;

     _RetChar(ReplyPacket.objectname);
            
}

/**************************************************************************
**
**   Get Novell Connection    0 < connection < 65535
**
***************************************************************************/

void FAR connection(ParamBlk FAR *parm)

{
     int station ;

     _segread(&segregs) ;
     segregs.es = segregs.ds ;

//   Get Station  
         
     inregs.x.ax=(0xdc00) ;
     _intdosx(&inregs, &outregs, &segregs) ;
     station=(outregs.x.ax & 0xff) ;

     _RetInt(station,5);
            
}

/**************************************************************************
**
**   Get Novell Node Address   Node Address in form AAAA11112222
**
***************************************************************************/

void FAR node_address(ParamBlk FAR *parm)

{
     int station,physical,i ;
     char tstation[12] = {'\0'} ;
     char astation[20] ;

     _segread(&segregs) ;
     segregs.es = segregs.ds ;

//   Get Station  
         
     inregs.x.ax=(0xdc00) ;
     _intdosx(&inregs, &outregs, &segregs) ;
     station=(outregs.x.ax & 0xff) ;
     
//   Get Physical Connection 

     inregs.x.ax=(0xee00) ;
     _intdosx(&inregs, &outregs, &segregs) ;
     physical=outregs.x.ax ;

//   Get username

     inregs.x.si=(int)&RequestPacket ;
     inregs.x.di=(int)&ReplyPacket ;
     inregs.x.ax= (0xe300) ;
     ReplyPacket.length=sizeof(ReplyPacket) ;
     RequestPacket.length=sizeof(RequestPacket) ;
     RequestPacket.function=22 ;
     RequestPacket.station=station ;
     _intdosx(&inregs, &outregs, &segregs) ;

     inregs.x.si=(int)&RequestPacket ;
     inregs.x.di=(int)&map ;
     inregs.x.ax=0xe300 ;
     RequestPacket.length=sizeof(RequestPacket) ;
     RequestPacket.function=19 ;
     RequestPacket.station=station ;
     map.length=sizeof(map) ;
     _intdosx(&inregs, &outregs, &segregs) ;
     
     physical=map.host[0] & 0x00ff ;
     _itoa(physical, astation, 16);
     if (physical < 16) {
          astation[2] = '\0' ;
          astation[1] = astation[0] ;
          astation[0] = '0' ;
          };
                    
     strcat(tstation, astation);

     physical=map.host[1] & 0x00ff ;
     _itoa(physical, astation, 16);
     if (physical < 16) {
          astation[2] = '\0' ;
          astation[1] = astation[0] ;
          astation[0] = '0' ;
          };

     strcat(tstation, astation);
     
     physical=map.host[2] & 0x00ff ;
     _itoa(physical, astation, 16);
     if (physical < 16) {
          astation[2] = '\0' ;
          astation[1] = astation[0] ;
          astation[0] = '0' ;
          };
     strcat(tstation, astation);

     physical=map.host[3] & 0x00ff ;
     _itoa(physical, astation, 16);
     if (physical < 16) {
          astation[2] = '\0' ;
          astation[1] = astation[0] ;
          astation[0] = '0' ;
          };
     strcat(tstation, astation);

     physical=map.host[4] & 0x00ff ;
     _itoa(physical, astation, 16);
     if (physical < 16) {
          astation[2] = '\0' ;
          astation[1] = astation[0] ;
          astation[0] = '0' ;
          };
     strcat(tstation, astation);

     physical=map.host[5] & 0x00ff ;
     _itoa(physical, astation, 16);
     if (physical < 16) {
          astation[2] = '\0' ;
          astation[1] = astation[0] ;
          astation[0] = '0' ;
          };
     strcat(tstation, astation);

// Convert String to Upper Case
     
     for (i = 0; i < (int) strlen(tstation); i++ )
     {
          tstation[i] = toupper( tstation[i] ) ;
     };                         

     _RetChar(tstation);

}

FoxInfo myFoxInfo[] = {
     {"USER_NAME", (FPFI) user_name, 0, ""},
     {"CONNECTION", (FPFI) connection, 0, ""},
     {"NODE", (FPFI) node_address, 0, ""}
};
FoxTable _FoxTable = {
     (FoxTable FAR *) 0, sizeof(myFoxInfo)/sizeof(FoxInfo), myFoxInfo
};
